from phrase import settings as phrase_settings
from six import python_2_unicode_compatible
from six import text_type
import logging

logger = logging.getLogger(__name__)


@python_2_unicode_compatible
class PhraseDelegate:
    def __init__(self, name, trimmed=False):
        self.name = name
        self.trimmed = trimmed

    def __str__(self):
        logger.info("Rendering Phrase Key: %s" % (self.name))
        tmp_name = self.__normalized_name()
        return tmp_name

    def __normalized_name(self):
        normalized = "%sphrase_%s%s" % (
            phrase_settings.PHRASE_PREFIX,
            self.__safe_name(),
            phrase_settings.PHRASE_SUFFIX,
        )
        return normalized

    def __safe_name(self):
        name = self.__safer_name()
        name = name.strip()
        name = name.replace("<", "[[[[[[html_open]]]]]]")
        name = name.replace(">", "[[[[[[html_close]]]]]]")
        return name

    def __safer_name(self):
        if type(self.name) is text_type:
            return text_type(self.name)
        else:
            return text_type(self.name.literal)



from django.conf import settings

PHRASE_ENABLED = getattr(settings, "PHRASE_ENABLED", True)
PHRASE_ACCOUNT_ID = getattr(settings, "PHRASE_ACCOUNT_ID", "")
PHRASE_PROJECT_ID = getattr(settings, "PHRASE_PROJECT_ID", "")
PHRASE_DATACENTER = getattr(settings, "PHRASE_DATACENTER", "")
PHRASE_PREFIX = getattr(settings, "PHRASE_PREFIX", "{{__")
PHRASE_SUFFIX = getattr(settings, "PHRASE_SUFFIX", "__}}")


def template_string_if_valid():
    try:  # does not throw an exception as of 1.11 yet
        value = settings.TEMPLATE_STRING_IF_INVALID
        if value:
            return value
    except:
        pass

    for templ in settings.TEMPLATES:
        if "OPTIONS" in templ:
            templ_options = templ["OPTIONS"]
            if "string_if_invalid" in templ_options:
                return templ_options["string_if_invalid"]

    return ""



from django.template import Context, Template
from django.test import TestCase


def render_template(content, **context_args):
    """Create a template that loads the `phrase_i18n` tags and adds the given content."""
    template = Template("{% load phrase_i18n %}" + content)
    return template.render(Context(context_args))


class TemplateTagsTest(TestCase):
    """Test all template tags."""

    def test_empty_template(self):
        self.assertEqual(render_template(""), "")

    def test_phrase_javascript(self):
        self.assertHTMLEqual(
            render_template("{% phrase_javascript %}"),
            """<script>
                window.PHRASEAPP_CONFIG = {
                    accountId: '',
                    projectId: '',
                    autoLowercase :false,
                    };
                (function() {
                var phrasejs = document.createElement('script');
                phrasejs.type = 'module';
                phrasejs.async = true;
                phrasejs.src = 'https://d2bgdldl6xit7z.cloudfront.net/latest/ice/index.js'
                var s = document.getElementsByTagName('script')[0]; s.parentNode.insertBefore(phrasejs, s); \
                })();
            </script>""",
        )



from six import string_types

# Django >= 2.1 uses TokenType enum.
from django.template.base import TokenType

TOKEN_MAPPING = None
TOKEN_TEXT = TokenType.TEXT
TOKEN_VAR = TokenType.VAR
TOKEN_BLOCK = TokenType.BLOCK


def is_string_type(value):
    """Return whether given value is of a known string type."""
    return isinstance(value, string_types)
